"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const chart_js_1 = require("chart.js");
class SolidGauge extends chart_js_1.DoughnutController {
    constructor() {
        super(...arguments);
        this._outerArcWidth = 0;
        this._innerArcWidth = 0;
        this._arcThickness = 0;
        this._arcThicknessHalf = 0;
        this._arcY = 0;
        this._distanceFromCanvasEdgeX = 0;
        this._box = {
            id: "solidgauge-box",
            position: "bottom",
            draw: () => { },
            weight: 1000,
            fullSize: false,
            width: 0,
            height: 0,
            top: 0,
            left: 0,
            right: 0,
            bottom: 0,
            responsive: false,
            isHorizontal: () => true,
            update: () => { },
            options: {},
        };
    }
    draw() {
        const chart = this.chart;
        const { text, textFont, textColor, subtext, subtextFont, subtextColor, min, max, } = chart.options;
        const { ctx } = chart;
        const { height: canvasHeight, width: canvasWidth } = ctx.canvas;
        const canvasWidthHalf = canvasWidth / 2;
        const datasetMeta = chart.getDatasetMeta(0);
        const { innerRadius, outerRadius, y: arcY } = datasetMeta.data[0];
        this._arcY = arcY;
        this._outerArcWidth = Math.round(this.outerRadius * 2);
        this._innerArcWidth = Math.round(this.innerRadius * 2);
        this._distanceFromCanvasEdgeX = Math.round((canvasWidth - this._outerArcWidth) / 2);
        this._arcThickness = Math.round(outerRadius - innerRadius);
        this._arcThicknessHalf = Math.round(this._arcThickness / 2);
        ctx.restore();
        ctx.textAlign = "start";
        ctx.textBaseline = "middle";
        ctx.save();
        if (text) {
            const textFontSize = (this._innerArcWidth / 70).toFixed(2);
            ctx.font = textFont || `bold ${textFontSize}em sans-serif`;
            const textDimensions = ctx.measureText(text);
            const textHeight = textDimensions.actualBoundingBoxAscent +
                textDimensions.actualBoundingBoxDescent;
            const textX = Math.round(canvasWidthHalf - textDimensions.width / 2);
            let textY = Math.round(arcY - textHeight);
            let subtextY = textY + textHeight;
            if (subtextY > canvasHeight) {
                const difference = subtextY - canvasHeight;
                textY -= difference;
                subtextY -= difference;
            }
            ctx.fillStyle = textColor || "#000";
            ctx.fillText(text, textX, textY);
            ctx.restore();
            if (subtext) {
                const subtextFontSize = (Number(textFontSize) / 2).toFixed(2);
                ctx.font =
                    subtextFont || `bold ${subtextFontSize}em sans-serif`;
                const subtextDimensions = ctx.measureText(subtext);
                const subtextX = Math.round(canvasWidthHalf - subtextDimensions.width / 2);
                ctx.fillStyle = subtextColor || "#000";
                ctx.fillText(subtext, subtextX, subtextY);
                ctx.restore();
            }
        }
        ctx.save();
        if (min || max) {
            const heightMinOrMax = this._drawMinMax();
            const newHeight = heightMinOrMax + this._arcThickness / 2;
            const solidGaugeBox = this.chart.boxes.find((box) => box.id === "solidgauge-box");
            if (solidGaugeBox) {
                if (solidGaugeBox.height < newHeight) {
                    solidGaugeBox.height = newHeight;
                }
            }
            else {
                chart_js_1.layouts.addBox(this.chart, Object.assign(Object.assign({}, this._box), { height: newHeight }));
            }
        }
        super.draw();
    }
    _drawMinMax() {
        const chart = this.chart;
        const { ctx } = chart;
        const { min, max, minMaxFont, minMaxColor } = chart.options;
        const { width: canvasWidth } = ctx.canvas;
        const minIsNumber = typeof min === "number";
        const maxIsNumber = typeof max === "number";
        const minString = minIsNumber ? `${min}` : "0";
        const maxString = maxIsNumber ? `${max}` : "100";
        const minMaxFontSize = (this._arcThickness / 70).toFixed(2);
        ctx.font = minMaxFont || `normal ${minMaxFontSize}em sans-serif`;
        const minOrMaxDimensions = ctx.measureText(minString || maxString);
        const minOrMaxHeight = minOrMaxDimensions.actualBoundingBoxAscent +
            minOrMaxDimensions.actualBoundingBoxDescent;
        const minMaxY = Math.round(this._arcY + minOrMaxHeight + minOrMaxHeight / 2);
        const minDimensions = ctx.measureText(minString);
        const minX = Math.round(this._distanceFromCanvasEdgeX +
            this._arcThicknessHalf -
            minDimensions.width / 2);
        const maxDimensions = ctx.measureText(maxString);
        const maxX = Math.round(canvasWidth -
            this._distanceFromCanvasEdgeX -
            this._arcThicknessHalf -
            maxDimensions.width / 2);
        ctx.fillStyle = minMaxColor || "#000";
        ctx.fillText(minString, minX, minMaxY);
        ctx.fillText(maxString, maxX, minMaxY);
        return minOrMaxHeight;
    }
}
exports.default = SolidGauge;
SolidGauge.id = "solidgauge";
chart_js_1.Chart.register(SolidGauge);
